/*
 * kioskgui.cpp
 *
 *   Copyright (C) 2003,2004 Waldo Bastian <bastian@kde.org>
 *   Copyright (C) 2009 Ian Reinhart Geiser <geiseri@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License versin 2 as
 *   published by the Free Software Foundation.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.
 */

#include <config.h>

#include "kioskgui.h"

#include <QDir>
#include <QLineEdit>
#include <QObject>
#include <QStackedWidget>
#include <QPushButton>
#include <QPixmap>
#include <QMenu>

#include <KDE/KApplication>
#include <KDE/KActionCollection>
#include <KDE/KAction>
#include <KDE/KComboBox>
#include <KDE/KConfig>
#include <KDE/KFileDialog>
#include <KDE/KGlobal>
#include <KDE/KIconLoader>
#include <KDE/KListWidget>
#include <KDE/KLocale>
#include <KDE/KMenuBar>
#include <KDE/KMessageBox>
#include <KDE/KStatusBar>
#include <kstdaccel.h>
#include <KDE/KStandardAction>
#include <KDE/KStandardDirs>
#include <KDE/KUrlRequester>
#include <KDE/KToggleAction>

#include <kdebug.h>

#include "profilePropsPage.h"
#include "componentSelectionPage.h"
#include "desktopComponent.h"
#include "menueditComponent.h"
#include "screensaverComponent.h"
#include "panelComponent.h"
#include "filetypeeditComponent.h"
#include "componentPage.h"
#include "pageWidget.h"
#include "userManagement.h"
#include "kioskConfigDialog.h"
#include "kioskdata.h"
#include "kioskrun.h"
#include "kiosksync.h"

KioskGui::KioskGui()
    : KXmlGuiWindow(), m_profileSelectionPage(0),
    m_componentSelectionPage(0), m_componentPage(0), m_profileAssignPage(0),
    m_profilePropsPage(0), m_data(0),
    m_profile(), m_componentData(0),m_page(PAGE_PROFILE_SELECTION)
{
    m_run = new KioskRun(this);
    m_view = new MainView(this);

    setupActions();

    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);

    createGUI();

    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();

    connect(m_view->pbDiscard, SIGNAL(clicked()), this, SLOT(discardPage()));
    connect(m_view->pbFinished, SIGNAL(clicked()), this, SLOT(finishedPage()));
    connect(m_view->pbHelp, SIGNAL(clicked()), this, SLOT(help()));

    m_view->pbHelp->setGuiItem(KStandardGuiItem::Help);

    KConfigGroup config( KGlobal::config(), "General");
    m_profile = config.readEntry("CurrentProfile", "default");
    m_component = config.readEntry("CurrentComponent");

    selectPage(PAGE_PROFILE_SELECTION);

    QTimer::singleShot(0, this, SLOT(slotCheckEtcSkel()));

}

KioskGui::~KioskGui()
{
    delete m_data;
}

void KioskGui::slotCheckEtcSkel()
{
    QString etcSkel = "/etc/skel/" KDE_DEFAULT_HOME;
    KioskSync skelDir;
    skelDir.addDir(etcSkel, KUrl());
    QStringList skelFiles = skelDir.listFiles();
    if (!skelFiles.isEmpty())
    {
        KMessageBox::informationList(this,
            i18n("<qt>Your system contains KDE configuration settings in the "
                 "skeleton directory <b>%1</b>. These files are copied to the "
                 "personal KDE settings directory of newly created users.<p>"
                 "This may interfere with the correct operation of user profiles.<p>"
                 "Unless a setting has been locked down, settings that have been copied "
                 "to the personal KDE settings directory of a user will override "
                 "a default setting configured in a profile.<p>"
                 "<b>If this is not the intended behavior, please remove the offending "
                 "files from the skeleton folder on all systems that you want to "
                 "administer with user profiles.</b><p>"
                 "The following files were found under <b>%2</b>:", etcSkel, etcSkel),
            skelFiles,
            QString(),
            "etc_skel_warning");
    }
}

void KioskGui::setSubCaption(const QString &subCaption)
{
    m_view->subCaptionLabel->setText("<h1>"+subCaption+"</h1>");
}

void KioskGui::setMainCaption( const QString &caption )
{
    m_view->captionLabel->setText("<h1>" + caption + "<h1>");
}

void KioskGui::setupActions()
{
    actionCollection()->addAction( KStandardAction::Quit, this, SLOT( close() ) );
    KStandardAction::preferences(this, SLOT(slotConfig()), actionCollection());


    m_uploadAction = actionCollection()->addAction("upload_all");
    m_uploadAction->setText(i18n( "Upload &All Profiles" ));
    connect(m_uploadAction, SIGNAL(triggered()), SLOT( uploadAllProfiles() ));

    updateActions();
}

void KioskGui::updateActions()
{
    KConfigGroup config( KGlobal::config(), "General");

    bool uploadEnabled = !config.readEntry("uploadURL").isEmpty();

    m_uploadAction->setEnabled(uploadEnabled);
}

void KioskGui::saveProperties(KConfigGroup& /*config*/ )
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored
}

void KioskGui::readProperties(const KConfigGroup & /*config*/ )
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'
}

void KioskGui::slotAddProfile()
{
    selectPage(PAGE_PROFILE_NEW);
}

void KioskGui::slotProfileProperties()
{
    selectPage(PAGE_PROFILE_PROPS);
}

void KioskGui::slotProfileSetup()
{
    selectPage(PAGE_COMPONENT_SELECTION);
}

void KioskGui::slotDeleteProfile(QTreeWidgetItem *item)
{
    if (!m_profileSelectionPage)
       return;
    if (!item)
       item = m_profileSelectionPage->listProfile->currentItem();
    if (!item)
       return;
    QString profile = item->text(0);
    int result = KMessageBox::warningContinueCancel(this,
               i18n("<qt>You are about to delete the profile <b>%1</b>.<p>"
                    "Are you sure you want to do this?", profile),
               QString::null, KGuiItem(i18n("Delete"),"editdelete"));
    if (result == KMessageBox::Continue)
    {
       if (KioskRun::self()->deleteProfile(profile))
       {
          delete item;

          item = m_profileSelectionPage->listProfile->currentItem();
          if (item)
             m_profileSelectionPage->listProfile->setCurrentItem(item);
       }
    }
}

void KioskGui::slotManageUsers()
{
    selectPage(PAGE_PROFILE_ASSIGN);
}


void KioskGui::selectPage(Page page)
{
    if( m_page ==  PAGE_COMPONENT_SELECTION )
    {
        m_component = m_componentSelectionPage->currentComponent();
        m_componentData = m_data->m_componentData[m_component];
        if (!m_componentData)
           return;
    }
    if( m_page ==  PAGE_PROFILE_SELECTION && m_profileSelectionPage )
    {
        m_profile = m_profileSelectionPage->listProfile->currentItem() ?
                    m_profileSelectionPage->listProfile->currentItem()->text(0) : QString();

        KConfigGroup config( KGlobal::config(), "General");
        config.writeEntry("CurrentProfile", m_profile);
        config.sync();

        QString description;
        QString installDir;
        QString installUser;

        KioskRun::self()->getProfileInfo(m_profile, description, installDir, installUser);

        QStringList kdeDirs;
        kdeDirs << installDir;
        m_run->setKdeDirs(kdeDirs);
        m_run->setUser(installUser);
    }
    if( m_page ==  PAGE_PROFILE_NEW )
    {
       m_profile = m_profilePropsPage->profile();
    }
    if( m_page ==  PAGE_PROFILE_PROPS )
    {
       m_profile = m_profilePropsPage->profile();
    }

    m_page = page;

    if (( m_page == PAGE_PROFILE_SELECTION) && !m_profileSelectionPage)
    {
       m_profileSelectionPage = new ProfileSelectionPageUI(m_view->widgetStack);
       connect(m_profileSelectionPage->buttonDelete, SIGNAL(clicked()),
               this, SLOT(slotDeleteProfile()));
       connect(m_profileSelectionPage->buttonAdd, SIGNAL(clicked()),
               this, SLOT(slotAddProfile()));
       connect(m_profileSelectionPage->buttonProperty, SIGNAL(clicked()),
               this, SLOT(slotProfileProperties()));
       connect(m_profileSelectionPage->buttonSetup, SIGNAL(clicked()),
               this, SLOT(slotProfileSetup()));
       connect(m_profileSelectionPage->buttonUsers, SIGNAL(clicked()),
               this, SLOT(slotManageUsers()));

       m_pageMapping[PAGE_PROFILE_SELECTION] = m_view->widgetStack->insertWidget(PAGE_PROFILE_SELECTION,m_profileSelectionPage);
       m_profileSelectionPage->listProfile->setFocus();
    }
    else if ((m_page == PAGE_COMPONENT_SELECTION) && !m_componentSelectionPage)
    {
       m_data = new KioskData;
       if (!m_data->load())
       {
          KMessageBox::error(this, m_data->errorMsg(), i18n("Error accessing Kiosk data"));
       }

       m_componentSelectionPage = new ComponentSelectionPage(m_data, m_view->widgetStack);
       connect(m_componentSelectionPage, SIGNAL(componentActivated()), this, SLOT(slotComponentSelection()));
       m_componentSelectionPage->setCurrentComponent(m_component);

       m_pageMapping[PAGE_COMPONENT_SELECTION] = m_view->widgetStack->insertWidget(PAGE_COMPONENT_SELECTION, m_componentSelectionPage);
       m_componentSelectionPage->listComponent->setFocus();
    }
    else if (m_page == PAGE_PROFILE_NEW)
    {
       delete m_profilePropsPage;
       m_profilePropsPage = new ProfilePropsPage(m_view->widgetStack, QString() );

       m_pageMapping[PAGE_PROFILE_NEW] =  m_view->widgetStack->insertWidget(PAGE_PROFILE_NEW,m_profilePropsPage->widget());
       m_profilePropsPage->setFocus();
    }
    else if (m_page == PAGE_PROFILE_PROPS)
    {
       delete m_profilePropsPage;
       m_profilePropsPage = new ProfilePropsPage(m_view->widgetStack, m_profile);

       m_pageMapping[PAGE_PROFILE_PROPS] = m_view->widgetStack->insertWidget(PAGE_PROFILE_PROPS,m_profilePropsPage->widget());
       m_profilePropsPage->setFocus();
    }
    else if ((m_page == PAGE_PROFILE_ASSIGN) && !m_profileAssignPage)
    {
       m_profileAssignPage = new UserManagementPage(m_view->widgetStack);

       m_pageMapping[PAGE_PROFILE_ASSIGN] = m_view->widgetStack->insertWidget(PAGE_PROFILE_ASSIGN,m_profileAssignPage->widget());
       m_profileAssignPage->setFocus();
    }
    else if (m_page == PAGE_COMPONENT)
    {
       delete m_componentPage;

       Component *component = 0;
       if (m_component == "kdesktop")
          component = new DesktopComponent;
       else if (m_component == "kdemenu")
          component = new MenuEditComponent;
       else if (m_component == "screensaver")
          component = new ScreenSaverComponent;
       else if (m_component == "kicker")
          component = new PanelComponent;
       else if (m_component == "filetypes")
          component = new FileTypeEditComponent;

       m_componentPage = new ComponentPage(m_componentData, component, m_profile, m_view->widgetStack);
       m_pageMapping[PAGE_COMPONENT] = m_view->widgetStack->insertWidget(PAGE_COMPONENT,m_componentPage->widget());
       m_componentPage->setFocus();
    }

    if( m_page == PAGE_PROFILE_SELECTION )
    {
       setSubCaption(i18n("Main Menu"));
       setMainCaption(i18n("Main Menu"));
    }
    if( m_page ==  PAGE_PROFILE_NEW )
    {
       setSubCaption(i18n("Add New Profile"));
       setMainCaption(i18n("Add New Profile"));
    }
    if( m_page == PAGE_PROFILE_PROPS )
    {
       setSubCaption(i18n("Profile Properties"));
       setMainCaption(i18n("Profile Properties"));
    }
    if( m_page ==  PAGE_COMPONENT_SELECTION )
    {
       setSubCaption(i18n("Setup Profile \"%1\"",m_profile));
       setMainCaption(i18n("Setup Profile"));
    }
    if( m_page ==  PAGE_COMPONENT )
    {
       setSubCaption(m_componentPage->subCaption());
    }
    if( m_page ==  PAGE_PROFILE_ASSIGN )
    {
       setSubCaption(i18n("Assign Profiles"));
    }

    m_view->widgetStack->setCurrentIndex(m_pageMapping[m_page]);
    loadPage(m_page);

    updateButtons();
}

void KioskGui::updateButtons()
{
      if( m_page ==  PAGE_PROFILE_SELECTION)
      {
         m_view->pbDiscard->hide();
         m_view->pbFinished->hide();
      }

      if( m_page ==   PAGE_COMPONENT_SELECTION)
      {
         m_view->pbDiscard->show();
         m_view->pbFinished->show();
      }

      if( m_page ==   PAGE_COMPONENT)
      {
         m_view->pbDiscard->hide();
         m_view->pbFinished->show();
      }

      if( m_page ==   PAGE_PROFILE_NEW)
      {
         m_view->pbDiscard->show();
         m_view->pbFinished->show();
      }

      if( m_page ==   PAGE_PROFILE_PROPS)
      {
         m_view->pbDiscard->show();
         m_view->pbFinished->show();
      }

      if( m_page ==   PAGE_PROFILE_ASSIGN)
      {
         m_view->pbDiscard->show();
         m_view->pbFinished->show();
      }

    if (m_page == PAGE_PROFILE_NEW)
    {
      m_view->pbDiscard->setGuiItem(KStandardGuiItem::Cancel);
      m_view->pbFinished->setGuiItem(KStandardGuiItem::Add);
    }
    if (m_page == PAGE_COMPONENT )
    {
      m_view->pbFinished->setGuiItem(KStandardGuiItem::Back);
    }
    else
    {
      m_view->pbDiscard->setGuiItem(KStandardGuiItem::Discard);
      m_view->pbFinished->setGuiItem(KStandardGuiItem::Save);
    }

}

void KioskGui::loadProfiles()
{
   m_profileSelectionPage->listProfile->clear();

   QStringList profiles = KioskRun::self()->allProfiles();

   for(QStringList::ConstIterator it = profiles.constBegin();
       it != profiles.constEnd(); ++it)
   {
      QString profile = *it;
      QString description;
      QString installUser;
      QString installDir;

      KioskRun::self()->getProfileInfo(profile, description, installDir, installUser);

      QTreeWidgetItem *item = new QTreeWidgetItem(m_profileSelectionPage->listProfile, QStringList() << profile << description);
      if (m_profile == profile)
         m_profileSelectionPage->listProfile->setCurrentItem(item);
   }
   if (!m_profileSelectionPage->listProfile->currentItem())
      m_profileSelectionPage->listProfile->setCurrentItem(m_profileSelectionPage->listProfile->topLevelItem(0));
}

void KioskGui::loadPage(Page page)
{
      if( page == PAGE_PROFILE_SELECTION )
      {
        loadProfiles();
      }
      else if( page == PAGE_COMPONENT )
      {
        m_componentPage->load();
      }
      else if( page == PAGE_PROFILE_NEW)
      {
        m_profilePropsPage->load();
      }
      else if( page == PAGE_PROFILE_PROPS )
      {
        m_profilePropsPage->load();
      }
      else if( page == PAGE_PROFILE_ASSIGN )
      {
        m_profileAssignPage->load();
      }
}

bool KioskGui::savePage(Page page)
{
      if( page ==  PAGE_COMPONENT_SELECTION)
      {
        return m_componentSelectionPage->save();
      }
      if( page ==  PAGE_COMPONENT )
      {
        if (m_componentPage)
           return m_componentPage->save();
      }
      if( page ==  PAGE_PROFILE_NEW )
      {
        if (m_profilePropsPage)
        {
           return m_profilePropsPage->save();
        }
      }
      if( page ==  PAGE_PROFILE_PROPS )
      {
        if (m_profilePropsPage)
        {
           return m_profilePropsPage->save();
        }
      }
      if( page ==  PAGE_PROFILE_ASSIGN)
      {
        if (m_profileAssignPage)
           return m_profileAssignPage->save();
      }
    return true;
}

void KioskGui::finishedPage(bool save)
{
    if( save )
    {
        if( !savePage( m_page ))
            return;
    }
    if (m_page == PAGE_PROFILE_NEW)
       selectPage(PAGE_PROFILE_SELECTION);
    else if (m_page == PAGE_PROFILE_PROPS)
       selectPage(PAGE_PROFILE_SELECTION);
    else if (m_page == PAGE_PROFILE_ASSIGN)
       selectPage(PAGE_PROFILE_SELECTION);
    else if (m_page == PAGE_COMPONENT_SELECTION)
    {
        if( !save )
	    m_componentSelectionPage->discard();
	selectPage(PAGE_PROFILE_SELECTION);
    }
    else if (m_page == PAGE_COMPONENT)
       selectPage(PAGE_COMPONENT_SELECTION);
}

void KioskGui::discardPage()
{
   finishedPage(false);
}

void KioskGui::slotComponentSelection()
{
       selectPage(PAGE_COMPONENT);
}

void KioskGui::slotConfig()
{
    KioskConfigDialog dlg(this);
    if (dlg.exec())
    {
       dlg.save();
       updateActions();
    }
}

bool KioskGui::queryClose()
{
    if (!savePage(m_page))
    {
       int result = KMessageBox::warningContinueCancel(this,
       		i18n("Your changes could not be saved, do you want to quit anyway?"),
       		QString::null,
       		KStandardGuiItem::quit());
       if (result == KMessageBox::Continue)
          return true;
       return false;
    }
    if (m_uploadAction->isEnabled())
    {
       KConfigGroup config( KGlobal::config(), "General");

       KUrl uploadUrl = config.readEntry("uploadURL");
       int result = KMessageBox::questionYesNo(this,
                    i18n("<qt>Do you want to upload the profiles to <b>%1</b> ?</qt>",uploadUrl.prettyUrl()));
       if (result == KMessageBox::Yes)
       {
          uploadAllProfiles();
       }
    }

    return true;
}

void KioskGui::uploadAllProfiles()
{
    KConfigGroup config( KGlobal::config(), "General");
   QString uploadPrefix = config.readEntry("uploadPrefix");
   QString uploadURL = config.readEntry("uploadURL");

   KioskSync sync(this);

   QStringList profiles = KioskRun::self()->allProfiles();

   foreach( QString profile, profiles)
   {
      QString description;
      QString installUser;
      QString installDir;

      KioskRun::self()->getProfileInfo(profile, description, installDir, installUser);

//      sync.addDir(installDir, KUrl("ftp://localhost/kde/profiles"));

      QString dir = installDir;
      if (dir.startsWith(uploadPrefix))
         dir = dir.mid(uploadPrefix.length());
      if (dir.startsWith("/"))
         dir = dir.mid(1);

      KUrl url(uploadURL);
      url.setPath(url.path( KUrl::AddTrailingSlash)+dir);
      sync.addDir(installDir, url);
   }

   if (sync.sync())
   {
      KMessageBox::information(this, i18n("<qt>All profiles have been successfully uploaded to <b>%1</b>",uploadURL));
   }
}

void KioskGui::uploadCurrentProfile()
{
}

void KioskGui::help()
{

}

#include "kioskgui.moc"
