/*
 *   userManagement.cpp
 *
 *   Copyright (C) 2004 Waldo Bastian <bastian@kde.org>
 *   Copyright (C) 2009 Ian Reinhart Geiser <geiseri@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 as
 *   published by the Free Software Foundation.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.
 */

#include "userManagement.h"

#include <unistd.h>
#include <grp.h>
#include <pwd.h>
#include <sys/types.h>

#include <QComboBox>
#include <QLabel>
#include <QPushButton>

#include <KDE/KApplication>
#include <KDE/KConfig>
#include <KDE/KListWidget>
#include <KDE/KLocale>
#include <KDE/KMessageBox>
#include <KDE/KGlobal>

#include "kioskrun.h"

UserManagementPage::UserManagementPage(QWidget* parent)
 : UserManagementUI(parent), m_widget(this)
{
  setWindowTitle(i18n("Assign Profiles"));
  listGroups->setDragEnabled(true);
  listGroups->setAcceptDrops(true);

//  actionButton(KDialogBase::Ok)->setFocus();

  connect(buttonAddGroup, SIGNAL(clicked()), this, SLOT(slotAddGroup()));
  connect(buttonDeleteGroup, SIGNAL(clicked()), this, SLOT(slotDeleteGroup()));
  connect(buttonAddUser, SIGNAL(clicked()), this, SLOT(slotAddUser()));
  connect(buttonDeleteUser, SIGNAL(clicked()), this, SLOT(slotDeleteUser()));

  connect(listGroups, SIGNAL(itemSelectionChanged()), this, SLOT(slotUpdateButtons()));
  connect(listUsers, SIGNAL(itemSelectionChanged()), this, SLOT(slotUpdateButtons()));
}

UserManagementPage::~UserManagementPage()
{
}

void UserManagementPage::load()
{
   listGroups->clear();
   listUsers->clear();

   m_allProfiles = KioskRun::self()->allProfiles();
   m_allProfiles.sort();

   KioskRun::ProfileMapping groups;
   KioskRun::ProfileMapping users;
   QStringList groupOrder;

   KioskRun::self()->getUserProfileMappings(groups, users, groupOrder);

   for ( QStringList::ConstIterator it = groupOrder.constBegin();
         it != groupOrder.constEnd(); ++it )
   {
      QString group = *it;
      QString profile = groups[group].join(",");
      new QTreeWidgetItem(listGroups, QStringList() << group << profile);
   }

   for ( KioskRun::ProfileMapping::Iterator it = users.begin();
         it != users.end(); ++it )
   {
      QString user = it.key();
      QString profile = it->join(",");
      new QTreeWidgetItem(listUsers, QStringList() << user << profile);
   }
   slotUpdateButtons();
}

void UserManagementPage::slotUpdateButtons()
{
   buttonDeleteGroup->setEnabled(listGroups->currentItem() != 0);
   buttonDeleteUser->setEnabled(listUsers->currentItem() != 0);
}

bool UserManagementPage::save()
{
   KioskRun::ProfileMapping groups;
   KioskRun::ProfileMapping users;
   QStringList groupOrder;

   for( int idx = 0; idx < listGroups->topLevelItemCount(); ++idx )
   {
      QTreeWidgetItem *item = listGroups->topLevelItem(idx);
      QString group = item->text(0);
      QStringList profiles = item->text(1).split(',');
      groups.insert(group, profiles);
      groupOrder.prepend(group);
   }

   for( int idx = 0; idx < listUsers->topLevelItemCount(); ++idx )
   {
      QTreeWidgetItem *item = listUsers->topLevelItem(idx);
      QString user = item->text(0);
      QStringList profiles = item->text(1).split(',');
      users.insert(user, profiles);
   }

   return KioskRun::self()->setUserProfileMappings(groups, users, groupOrder);
}

void UserManagementPage::slotAddGroup()
{
   KConfigGroup config( KGlobal::config(), "General");
   QStringList groupBlacklist = config.readEntry("GroupBlacklist", QStringList());

   m_allGroups.clear();
   setgrent();
   for (struct group *grp; (grp = getgrent()); )
   {
       QString group = QString::fromUtf8(grp->gr_name);
       if (!groupBlacklist.contains(group))
           m_allGroups.append(group);
   }
   endgrent();
   m_allGroups.sort();

   KDialog dlg(this );
   dlg.setButtons(KDialog::Ok|KDialog::Cancel);
   dlg.setDefaultButton(KDialog::Ok);
   dlg.setModal( true );
   dlg.setCaption( i18n("Add Group Policy"));
   UserManagementGroupUI *wid = new UserManagementGroupUI(&dlg);
   wid->labelCaption->setFixedSize(wid->labelCaption->sizeHint());
   wid->comboGroup->addItems(m_allGroups);
   wid->comboProfile->addItems(m_allProfiles);
   wid->setFixedSize(wid->sizeHint());
   dlg.setMainWidget(wid);
   dlg.setFixedSize(dlg.sizeHint());
   while (dlg.exec() == KDialog::Accepted)
   {
      QString group = wid->comboGroup->currentText();
      QString profile = wid->comboProfile->currentText();

      // Check for dupes
      QTreeWidgetItem *item = 0;
      for( int idx = 0; idx < listGroups->topLevelItemCount(); ++idx )
      {
         item = listGroups->topLevelItem(idx);
         if (item->text(0) == group)
            break;
      }
      if (item)
      {
         int result = KMessageBox::warningContinueCancel(this,
                   i18n("<qt>You already have a profile defined for group <b>%1</b>. "
                        "Do you want to replace it?</qt>", group),
                   i18n("Duplicate Warning"),
                   KGuiItem( i18n("&Replace") ));
         if (result != KMessageBox::Continue)
            continue; // Go back to edit dialog
         delete item;
      }

      item = new QTreeWidgetItem(listGroups, QStringList() << group << profile);
      listGroups->setCurrentItem(item);
      slotUpdateButtons();
      return;
   }
}

void UserManagementPage::slotDeleteGroup()
{
   QTreeWidgetItem *item = listGroups->currentItem();
   if (!item)
      return;

   delete item;

   item = listGroups->currentItem();
   if (item)
      listGroups->setCurrentItem(item);
   slotUpdateButtons();
}

void UserManagementPage::slotAddUser()
{
   KConfigGroup config( KGlobal::config(),"General");
   int minUID = config.readEntry("FirstUIDShown", 500);

   m_allUsers.clear();
   setpwent();
   for (struct passwd *user; (user = getpwent()); )
   {
       if ((user->pw_uid >= (uid_t) minUID) || (user->pw_uid == 0))
           m_allUsers.append(QString::fromUtf8(user->pw_name));
   }
   endpwent();
   m_allUsers.sort();

   KDialog dlg(this);
   dlg.setCaption( i18n("Add User Policy") );
   dlg.setButtons(KDialog::Ok|KDialog::Cancel);
   dlg.setDefaultButton( KDialog::Ok );
   dlg.setModal( true );

   UserManagementUserUI *wid = new UserManagementUserUI(&dlg);
   wid->labelCaption->setFixedSize(wid->labelCaption->sizeHint());
   wid->comboUser->addItems(m_allUsers);
   wid->comboProfile->addItems(m_allProfiles);
   wid->setFixedSize(wid->sizeHint());
   dlg.setMainWidget(wid);
   dlg.setFixedSize(dlg.sizeHint());
   while (dlg.exec() == KDialog::Accepted)
   {
      QString user = wid->comboUser->currentText();
      QString profile = wid->comboProfile->currentText();

      // Check for dupes
      QTreeWidgetItem *item = 0;
      for( int idx = 0; idx < listUsers->topLevelItemCount(); ++idx )
      {
         item = listUsers->topLevelItem(idx);

         if (item->text(0) == user)
            break;
      }
      if (item)
      {
         int result = KMessageBox::warningContinueCancel(this,
                   i18n("<qt>You already have a profile defined for user <b>%1</b>. "
                        "Do you want to replace it?</<qt>", user),
                   i18n("Duplicate Warning") ,
                   KGuiItem( i18n("&Replace")) );
         if (result != KMessageBox::Continue)
            continue; // Go back to edit dialog
         delete item;
      }

      item = new QTreeWidgetItem(listUsers, QStringList() << user << profile);
      listUsers->setCurrentItem(item);
      slotUpdateButtons();
      return;
   }
}

void UserManagementPage::slotDeleteUser()
{
   QTreeWidgetItem *item = listUsers->currentItem();
   if (!item)
      return;

   delete item;

   item = listUsers->currentItem();
   if (item)
      listUsers->setCurrentItem(item);
   slotUpdateButtons();
}

void UserManagementPage::setFocus()
{
   // TODO
}

QString UserManagementPage::subCaption()
{
   return i18n("Assign Profiles");
}

#include "userManagement.moc"
